
#include "menu.h"

void menu_draw(menu_t *menu);

int menu_display(menu_t *menu, menu_input (*input)(void))
{
    menu_state_init(menu);
    while(!menu_state_advance(menu, input()))
        ;
    return menu->selection;
}

void menu_state_init(menu_t *menu)
{
    menu->selection = 0;
    menu_draw(menu);
}

int menu_state_advance(menu_t *menu, menu_input input)
{
    switch(input) {
    case MENU_INPUT_CANCEL:
        menu->selection = -1;
        return 1;
    case MENU_INPUT_ENTER: {
        menu_entry_t *entry = &menu->entries[menu->selection];

        if(entry->type == MENU_TYPE_NORMAL)
	    return 1;
	if(entry->type == MENU_TYPE_CALLBACK) {
	    int cbr = entry->data.callback.cb(entry->data.callback.data);
	    menu_draw(menu);
            return !cbr;
	}
        return 0;
    }
    case MENU_INPUT_NEXT:
        menu->selection = (menu->selection+1) % menu->num_entries;
        break;
    case MENU_INPUT_PREV:
        if(menu->selection)
            menu->selection = (menu->selection-1) % menu->num_entries;
        else
            menu->selection = menu->num_entries-1;
        break;
    }

    menu_draw(menu);

    return 0;
}

void menu_draw(menu_t *menu)
{
    int offset = (menu->selection / MENU_PAGE_ITEMS) * MENU_PAGE_ITEMS;
    int end = offset + MENU_PAGE_ITEMS;
    if(end >= menu->num_entries)
        end = menu->num_entries;

    menu->write_begin();
    for(int index = offset, line = 0; index < end; index++, line++)
        menu->write(line, index == menu->selection, &menu->entries[index]);
}
